"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// ** This file is automatically generated by gapic-generator-typescript. **
// ** https://github.com/googleapis/gapic-generator-typescript **
// ** All changes to this file may be overwritten. **
Object.defineProperty(exports, "__esModule", { value: true });
const gax = require("google-gax");
const path = require("path");
const gapicConfig = require("./device_manager_client_config.json");
const version = require('../../../package.json').version;
/**
 *  Internet of Things (IoT) service. Securely connect and manage IoT devices.
 * @class
 * @memberof v1
 */
class DeviceManagerClient {
    /**
     * Construct an instance of DeviceManagerClient.
     *
     * @param {object} [options] - The configuration object. See the subsequent
     *   parameters for more details.
     * @param {object} [options.credentials] - Credentials object.
     * @param {string} [options.credentials.client_email]
     * @param {string} [options.credentials.private_key]
     * @param {string} [options.email] - Account email address. Required when
     *     using a .pem or .p12 keyFilename.
     * @param {string} [options.keyFilename] - Full path to the a .json, .pem, or
     *     .p12 key downloaded from the Google Developers Console. If you provide
     *     a path to a JSON file, the projectId option below is not necessary.
     *     NOTE: .pem and .p12 require you to specify options.email as well.
     * @param {number} [options.port] - The port on which to connect to
     *     the remote host.
     * @param {string} [options.projectId] - The project ID from the Google
     *     Developer's Console, e.g. 'grape-spaceship-123'. We will also check
     *     the environment variable GCLOUD_PROJECT for your project ID. If your
     *     app is running in an environment which supports
     *     {@link https://developers.google.com/identity/protocols/application-default-credentials Application Default Credentials},
     *     your project ID will be detected automatically.
     * @param {function} [options.promise] - Custom promise module to use instead
     *     of native Promises.
     * @param {string} [options.apiEndpoint] - The domain name of the
     *     API remote host.
     */
    constructor(opts) {
        this._descriptors = { page: {}, stream: {}, longrunning: {} };
        this._terminated = false;
        // Ensure that options include the service address and port.
        const staticMembers = this.constructor;
        const servicePath = opts && opts.servicePath
            ? opts.servicePath
            : opts && opts.apiEndpoint
                ? opts.apiEndpoint
                : staticMembers.servicePath;
        const port = opts && opts.port ? opts.port : staticMembers.port;
        if (!opts) {
            opts = { servicePath, port };
        }
        opts.servicePath = opts.servicePath || servicePath;
        opts.port = opts.port || port;
        opts.clientConfig = opts.clientConfig || {};
        const isBrowser = typeof window !== 'undefined';
        if (isBrowser) {
            opts.fallback = true;
        }
        // If we are in browser, we are already using fallback because of the
        // "browser" field in package.json.
        // But if we were explicitly requested to use fallback, let's do it now.
        const gaxModule = !isBrowser && opts.fallback ? gax.fallback : gax;
        // Create a `gaxGrpc` object, with any grpc-specific options
        // sent to the client.
        opts.scopes = this.constructor.scopes;
        const gaxGrpc = new gaxModule.GrpcClient(opts);
        // Save the auth object to the client, for use by other methods.
        this.auth = gaxGrpc.auth;
        // Determine the client header string.
        const clientHeader = [`gax/${gaxModule.version}`, `gapic/${version}`];
        if (typeof process !== 'undefined' && 'versions' in process) {
            clientHeader.push(`gl-node/${process.versions.node}`);
        }
        else {
            clientHeader.push(`gl-web/${gaxModule.version}`);
        }
        if (!opts.fallback) {
            clientHeader.push(`grpc/${gaxGrpc.grpcVersion}`);
        }
        if (opts.libName && opts.libVersion) {
            clientHeader.push(`${opts.libName}/${opts.libVersion}`);
        }
        // Load the applicable protos.
        // For Node.js, pass the path to JSON proto file.
        // For browsers, pass the JSON content.
        const nodejsProtoPath = path.join(__dirname, '..', '..', 'protos', 'protos.json');
        const protos = gaxGrpc.loadProto(opts.fallback ? require('../../protos/protos.json') : nodejsProtoPath);
        // This API contains "path templates"; forward-slash-separated
        // identifiers to uniquely identify resources within the API.
        // Create useful helper objects for these.
        this._pathTemplates = {
            locationPathTemplate: new gaxModule.PathTemplate('projects/{project}/locations/{location}'),
            registryPathTemplate: new gaxModule.PathTemplate('projects/{project}/locations/{location}/registries/{registry}'),
            devicePathTemplate: new gaxModule.PathTemplate('projects/{project}/locations/{location}/registries/{registry}/devices/{device}'),
        };
        // Some of the methods on this service return "paged" results,
        // (e.g. 50 results at a time, with tokens to get subsequent
        // pages). Denote the keys used for pagination and results.
        this._descriptors.page = {
            listDeviceRegistries: new gaxModule.PageDescriptor('pageToken', 'nextPageToken', 'deviceRegistries'),
            listDevices: new gaxModule.PageDescriptor('pageToken', 'nextPageToken', 'devices'),
        };
        // Put together the default options sent with requests.
        const defaults = gaxGrpc.constructSettings('google.cloud.iot.v1.DeviceManager', gapicConfig, opts.clientConfig || {}, { 'x-goog-api-client': clientHeader.join(' ') });
        // Set up a dictionary of "inner API calls"; the core implementation
        // of calling the API is handled in `google-gax`, with this code
        // merely providing the destination and request information.
        this._innerApiCalls = {};
        // Put together the "service stub" for
        // google.cloud.iot.v1.DeviceManager.
        this.deviceManagerStub = gaxGrpc.createStub(opts.fallback
            ? protos.lookupService('google.cloud.iot.v1.DeviceManager')
            : // tslint:disable-next-line no-any
                protos.google.cloud.iot.v1.DeviceManager, opts);
        // Iterate over each of the methods that the service provides
        // and create an API call method for each.
        const deviceManagerStubMethods = [
            'createDeviceRegistry',
            'getDeviceRegistry',
            'updateDeviceRegistry',
            'deleteDeviceRegistry',
            'listDeviceRegistries',
            'createDevice',
            'getDevice',
            'updateDevice',
            'deleteDevice',
            'listDevices',
            'modifyCloudToDeviceConfig',
            'listDeviceConfigVersions',
            'listDeviceStates',
            'setIamPolicy',
            'getIamPolicy',
            'testIamPermissions',
            'sendCommandToDevice',
            'bindDeviceToGateway',
            'unbindDeviceFromGateway',
        ];
        for (const methodName of deviceManagerStubMethods) {
            const innerCallPromise = this.deviceManagerStub.then(stub => (...args) => {
                if (this._terminated) {
                    return Promise.reject('The client has already been closed.');
                }
                return stub[methodName].apply(stub, args);
            }, (err) => () => {
                throw err;
            });
            const apiCall = gaxModule.createApiCall(innerCallPromise, defaults[methodName], this._descriptors.page[methodName] ||
                this._descriptors.stream[methodName] ||
                this._descriptors.longrunning[methodName]);
            this._innerApiCalls[methodName] = (argument, callOptions, callback) => {
                return apiCall(argument, callOptions, callback);
            };
        }
    }
    /**
     * The DNS address for this API service.
     */
    static get servicePath() {
        return 'cloudiot.googleapis.com';
    }
    /**
     * The DNS address for this API service - same as servicePath(),
     * exists for compatibility reasons.
     */
    static get apiEndpoint() {
        return 'cloudiot.googleapis.com';
    }
    /**
     * The port for this API service.
     */
    static get port() {
        return 443;
    }
    /**
     * The scopes needed to make gRPC calls for every method defined
     * in this service.
     */
    static get scopes() {
        return [
            'https://www.googleapis.com/auth/cloud-platform',
            'https://www.googleapis.com/auth/cloudiot',
        ];
    }
    /**
     * Return the project ID used by this class.
     * @param {function(Error, string)} callback - the callback to
     *   be called with the current project Id.
     */
    getProjectId(callback) {
        if (callback) {
            this.auth.getProjectId(callback);
            return;
        }
        return this.auth.getProjectId();
    }
    /**
     * Creates a device registry that contains devices.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The project and cloud region where this device registry must be created.
     *   For example, `projects/example-project/locations/us-central1`.
     * @param {google.cloud.iot.v1.DeviceRegistry} request.deviceRegistry
     *   Required. The device registry. The field `name` must be empty. The server will
     *   generate that field from the device registry `id` provided and the
     *   `parent` field.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [DeviceRegistry]{@link google.cloud.iot.v1.DeviceRegistry}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    createDeviceRegistry(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            parent: request.parent || '',
        });
        return this._innerApiCalls.createDeviceRegistry(request, options, callback);
    }
    /**
     * Gets a device registry configuration.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device registry. For example,
     *   `projects/example-project/locations/us-central1/registries/my-registry`.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [DeviceRegistry]{@link google.cloud.iot.v1.DeviceRegistry}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    getDeviceRegistry(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.getDeviceRegistry(request, options, callback);
    }
    /**
     * Updates a device registry configuration.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {google.cloud.iot.v1.DeviceRegistry} request.deviceRegistry
     *   Required. The new values for the device registry. The `id` field must be empty, and
     *   the `name` field must indicate the path of the resource. For example,
     *   `projects/example-project/locations/us-central1/registries/my-registry`.
     * @param {google.protobuf.FieldMask} request.updateMask
     *   Required. Only updates the `device_registry` fields indicated by this mask.
     *   The field mask must not be empty, and it must not contain fields that
     *   are immutable or only set by the server.
     *   Mutable top-level fields: `event_notification_config`, `http_config`,
     *   `mqtt_config`, and `state_notification_config`.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [DeviceRegistry]{@link google.cloud.iot.v1.DeviceRegistry}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    updateDeviceRegistry(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            'device_registry.name': request.deviceRegistry.name || '',
        });
        return this._innerApiCalls.updateDeviceRegistry(request, options, callback);
    }
    /**
     * Deletes a device registry configuration.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device registry. For example,
     *   `projects/example-project/locations/us-central1/registries/my-registry`.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [Empty]{@link google.protobuf.Empty}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    deleteDeviceRegistry(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.deleteDeviceRegistry(request, options, callback);
    }
    /**
     * Creates a device in a device registry.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The name of the device registry where this device should be created.
     *   For example,
     *   `projects/example-project/locations/us-central1/registries/my-registry`.
     * @param {google.cloud.iot.v1.Device} request.device
     *   Required. The device registration details. The field `name` must be empty. The server
     *   generates `name` from the device registry `id` and the
     *   `parent` field.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [Device]{@link google.cloud.iot.v1.Device}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    createDevice(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            parent: request.parent || '',
        });
        return this._innerApiCalls.createDevice(request, options, callback);
    }
    /**
     * Gets details about a device.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device. For example,
     *   `projects/p0/locations/us-central1/registries/registry0/devices/device0` or
     *   `projects/p0/locations/us-central1/registries/registry0/devices/{num_id}`.
     * @param {google.protobuf.FieldMask} request.fieldMask
     *   The fields of the `Device` resource to be returned in the response. If the
     *   field mask is unset or empty, all fields are returned.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [Device]{@link google.cloud.iot.v1.Device}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    getDevice(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.getDevice(request, options, callback);
    }
    /**
     * Updates a device.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {google.cloud.iot.v1.Device} request.device
     *   Required. The new values for the device. The `id` and `num_id` fields must
     *   be empty, and the field `name` must specify the name path. For example,
     *   `projects/p0/locations/us-central1/registries/registry0/devices/device0`or
     *   `projects/p0/locations/us-central1/registries/registry0/devices/{num_id}`.
     * @param {google.protobuf.FieldMask} request.updateMask
     *   Required. Only updates the `device` fields indicated by this mask.
     *   The field mask must not be empty, and it must not contain fields that
     *   are immutable or only set by the server.
     *   Mutable top-level fields: `credentials`, `blocked`, and `metadata`
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [Device]{@link google.cloud.iot.v1.Device}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    updateDevice(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            'device.name': request.device.name || '',
        });
        return this._innerApiCalls.updateDevice(request, options, callback);
    }
    /**
     * Deletes a device.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device. For example,
     *   `projects/p0/locations/us-central1/registries/registry0/devices/device0` or
     *   `projects/p0/locations/us-central1/registries/registry0/devices/{num_id}`.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [Empty]{@link google.protobuf.Empty}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    deleteDevice(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.deleteDevice(request, options, callback);
    }
    /**
     * Modifies the configuration for the device, which is eventually sent from
     * the Cloud IoT Core servers. Returns the modified configuration version and
     * its metadata.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device. For example,
     *   `projects/p0/locations/us-central1/registries/registry0/devices/device0` or
     *   `projects/p0/locations/us-central1/registries/registry0/devices/{num_id}`.
     * @param {number} request.versionToUpdate
     *   The version number to update. If this value is zero, it will not check the
     *   version number of the server and will always update the current version;
     *   otherwise, this update will fail if the version number found on the server
     *   does not match this version number. This is used to support multiple
     *   simultaneous updates without losing data.
     * @param {Buffer} request.binaryData
     *   Required. The configuration data for the device.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [DeviceConfig]{@link google.cloud.iot.v1.DeviceConfig}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    modifyCloudToDeviceConfig(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.modifyCloudToDeviceConfig(request, options, callback);
    }
    /**
     * Lists the last few versions of the device configuration in descending
     * order (i.e.: newest first).
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device. For example,
     *   `projects/p0/locations/us-central1/registries/registry0/devices/device0` or
     *   `projects/p0/locations/us-central1/registries/registry0/devices/{num_id}`.
     * @param {number} request.numVersions
     *   The number of versions to list. Versions are listed in decreasing order of
     *   the version number. The maximum number of versions retained is 10. If this
     *   value is zero, it will return all the versions available.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [ListDeviceConfigVersionsResponse]{@link google.cloud.iot.v1.ListDeviceConfigVersionsResponse}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    listDeviceConfigVersions(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.listDeviceConfigVersions(request, options, callback);
    }
    /**
     * Lists the last few versions of the device state in descending order (i.e.:
     * newest first).
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device. For example,
     *   `projects/p0/locations/us-central1/registries/registry0/devices/device0` or
     *   `projects/p0/locations/us-central1/registries/registry0/devices/{num_id}`.
     * @param {number} request.numStates
     *   The number of states to list. States are listed in descending order of
     *   update time. The maximum number of states retained is 10. If this
     *   value is zero, it will return all the states available.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [ListDeviceStatesResponse]{@link google.cloud.iot.v1.ListDeviceStatesResponse}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    listDeviceStates(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.listDeviceStates(request, options, callback);
    }
    /**
     * Sets the access control policy on the specified resource. Replaces any
     * existing policy.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [Policy]{@link google.iam.v1.Policy}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    setIamPolicy(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            resource: request.resource || '',
        });
        return this._innerApiCalls.setIamPolicy(request, options, callback);
    }
    /**
     * Gets the access control policy for a resource.
     * Returns an empty policy if the resource exists and does not have a policy
     * set.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [Policy]{@link google.iam.v1.Policy}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    getIamPolicy(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            resource: request.resource || '',
        });
        return this._innerApiCalls.getIamPolicy(request, options, callback);
    }
    /**
     * Returns permissions that a caller has on the specified resource.
     * If the resource does not exist, this will return an empty set of
     * permissions, not a NOT_FOUND error.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [TestIamPermissionsResponse]{@link google.iam.v1.TestIamPermissionsResponse}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    testIamPermissions(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            resource: request.resource || '',
        });
        return this._innerApiCalls.testIamPermissions(request, options, callback);
    }
    /**
     * Sends a command to the specified device. In order for a device to be able
     * to receive commands, it must:
     * 1) be connected to Cloud IoT Core using the MQTT protocol, and
     * 2) be subscribed to the group of MQTT topics specified by
     *    /devices/{device-id}/commands/#. This subscription will receive commands
     *    at the top-level topic /devices/{device-id}/commands as well as commands
     *    for subfolders, like /devices/{device-id}/commands/subfolder.
     *    Note that subscribing to specific subfolders is not supported.
     * If the command could not be delivered to the device, this method will
     * return an error; in particular, if the device is not subscribed, this
     * method will return FAILED_PRECONDITION. Otherwise, this method will
     * return OK. If the subscription is QoS 1, at least once delivery will be
     * guaranteed; for QoS 0, no acknowledgment will be expected from the device.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.name
     *   Required. The name of the device. For example,
     *   `projects/p0/locations/us-central1/registries/registry0/devices/device0` or
     *   `projects/p0/locations/us-central1/registries/registry0/devices/{num_id}`.
     * @param {Buffer} request.binaryData
     *   Required. The command data to send to the device.
     * @param {string} request.subfolder
     *   Optional subfolder for the command. If empty, the command will be delivered
     *   to the /devices/{device-id}/commands topic, otherwise it will be delivered
     *   to the /devices/{device-id}/commands/{subfolder} topic. Multi-level
     *   subfolders are allowed. This field must not have more than 256 characters,
     *   and must not contain any MQTT wildcards ("+" or "#") or null characters.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [SendCommandToDeviceResponse]{@link google.cloud.iot.v1.SendCommandToDeviceResponse}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    sendCommandToDevice(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            name: request.name || '',
        });
        return this._innerApiCalls.sendCommandToDevice(request, options, callback);
    }
    /**
     * Associates the device with the gateway.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The name of the registry. For example,
     *   `projects/example-project/locations/us-central1/registries/my-registry`.
     * @param {string} request.gatewayId
     *   Required. The value of `gateway_id` can be either the device numeric ID or the
     *   user-defined device identifier.
     * @param {string} request.deviceId
     *   Required. The device to associate with the specified gateway. The value of
     *   `device_id` can be either the device numeric ID or the user-defined device
     *   identifier.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [BindDeviceToGatewayResponse]{@link google.cloud.iot.v1.BindDeviceToGatewayResponse}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    bindDeviceToGateway(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            parent: request.parent || '',
        });
        return this._innerApiCalls.bindDeviceToGateway(request, options, callback);
    }
    /**
     * Deletes the association between the device and the gateway.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The name of the registry. For example,
     *   `projects/example-project/locations/us-central1/registries/my-registry`.
     * @param {string} request.gatewayId
     *   Required. The value of `gateway_id` can be either the device numeric ID or the
     *   user-defined device identifier.
     * @param {string} request.deviceId
     *   Required. The device to disassociate from the specified gateway. The value of
     *   `device_id` can be either the device numeric ID or the user-defined device
     *   identifier.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is an object representing [UnbindDeviceFromGatewayResponse]{@link google.cloud.iot.v1.UnbindDeviceFromGatewayResponse}.
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    unbindDeviceFromGateway(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            parent: request.parent || '',
        });
        return this._innerApiCalls.unbindDeviceFromGateway(request, options, callback);
    }
    /**
     * Lists device registries.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The project and cloud region path. For example,
     *   `projects/example-project/locations/us-central1`.
     * @param {number} request.pageSize
     *   The maximum number of registries to return in the response. If this value
     *   is zero, the service will select a default size. A call may return fewer
     *   objects than requested. A non-empty `next_page_token` in the response
     *   indicates that more data is available.
     * @param {string} request.pageToken
     *   The value returned by the last `ListDeviceRegistriesResponse`; indicates
     *   that this is a continuation of a prior `ListDeviceRegistries` call and
     *   the system should return the next page of data.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is Array of [DeviceRegistry]{@link google.cloud.iot.v1.DeviceRegistry}.
     *   The client library support auto-pagination by default: it will call the API as many
     *   times as needed and will merge results from all the pages into this array.
     *
     *   When autoPaginate: false is specified through options, the array has three elements.
     *   The first element is Array of [DeviceRegistry]{@link google.cloud.iot.v1.DeviceRegistry} that corresponds to
     *   the one page received from the API server.
     *   If the second element is not null it contains the request object of type [ListDeviceRegistriesRequest]{@link google.cloud.iot.v1.ListDeviceRegistriesRequest}
     *   that can be used to obtain the next page of the results.
     *   If it is null, the next page does not exist.
     *   The third element contains the raw response received from the API server. Its type is
     *   [ListDeviceRegistriesResponse]{@link google.cloud.iot.v1.ListDeviceRegistriesResponse}.
     *
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    listDeviceRegistries(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            parent: request.parent || '',
        });
        return this._innerApiCalls.listDeviceRegistries(request, options, callback);
    }
    /**
     * Equivalent to {@link listDeviceRegistries}, but returns a NodeJS Stream object.
     *
     * This fetches the paged responses for {@link listDeviceRegistries} continuously
     * and invokes the callback registered for 'data' event for each element in the
     * responses.
     *
     * The returned object has 'end' method when no more elements are required.
     *
     * autoPaginate option will be ignored.
     *
     * @see {@link https://nodejs.org/api/stream.html}
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The project and cloud region path. For example,
     *   `projects/example-project/locations/us-central1`.
     * @param {number} request.pageSize
     *   The maximum number of registries to return in the response. If this value
     *   is zero, the service will select a default size. A call may return fewer
     *   objects than requested. A non-empty `next_page_token` in the response
     *   indicates that more data is available.
     * @param {string} request.pageToken
     *   The value returned by the last `ListDeviceRegistriesResponse`; indicates
     *   that this is a continuation of a prior `ListDeviceRegistries` call and
     *   the system should return the next page of data.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Stream}
     *   An object stream which emits an object representing [DeviceRegistry]{@link google.cloud.iot.v1.DeviceRegistry} on 'data' event.
     */
    listDeviceRegistriesStream(request, options) {
        request = request || {};
        const callSettings = new gax.CallSettings(options);
        return this._descriptors.page.listDeviceRegistries.createStream(this._innerApiCalls.listDeviceRegistries, request, callSettings);
    }
    /**
     * List devices in a device registry.
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The device registry path. Required. For example,
     *   `projects/my-project/locations/us-central1/registries/my-registry`.
     * @param {number[]} request.deviceNumIds
     *   A list of device numeric IDs. If empty, this field is ignored. Maximum
     *   IDs: 10,000.
     * @param {string[]} request.deviceIds
     *   A list of device string IDs. For example, `['device0', 'device12']`.
     *   If empty, this field is ignored. Maximum IDs: 10,000
     * @param {google.protobuf.FieldMask} request.fieldMask
     *   The fields of the `Device` resource to be returned in the response. The
     *   fields `id` and `num_id` are always returned, along with any
     *   other fields specified.
     * @param {google.cloud.iot.v1.GatewayListOptions} request.gatewayListOptions
     *   Options related to gateways.
     * @param {number} request.pageSize
     *   The maximum number of devices to return in the response. If this value
     *   is zero, the service will select a default size. A call may return fewer
     *   objects than requested. A non-empty `next_page_token` in the response
     *   indicates that more data is available.
     * @param {string} request.pageToken
     *   The value returned by the last `ListDevicesResponse`; indicates
     *   that this is a continuation of a prior `ListDevices` call and
     *   the system should return the next page of data.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Promise} - The promise which resolves to an array.
     *   The first element of the array is Array of [Device]{@link google.cloud.iot.v1.Device}.
     *   The client library support auto-pagination by default: it will call the API as many
     *   times as needed and will merge results from all the pages into this array.
     *
     *   When autoPaginate: false is specified through options, the array has three elements.
     *   The first element is Array of [Device]{@link google.cloud.iot.v1.Device} that corresponds to
     *   the one page received from the API server.
     *   If the second element is not null it contains the request object of type [ListDevicesRequest]{@link google.cloud.iot.v1.ListDevicesRequest}
     *   that can be used to obtain the next page of the results.
     *   If it is null, the next page does not exist.
     *   The third element contains the raw response received from the API server. Its type is
     *   [ListDevicesResponse]{@link google.cloud.iot.v1.ListDevicesResponse}.
     *
     *   The promise has a method named "cancel" which cancels the ongoing API call.
     */
    listDevices(request, optionsOrCallback, callback) {
        request = request || {};
        let options;
        if (typeof optionsOrCallback === 'function' && callback === undefined) {
            callback = optionsOrCallback;
            options = {};
        }
        else {
            options = optionsOrCallback;
        }
        options = options || {};
        options.otherArgs = options.otherArgs || {};
        options.otherArgs.headers = options.otherArgs.headers || {};
        options.otherArgs.headers['x-goog-request-params'] = gax.routingHeader.fromParams({
            parent: request.parent || '',
        });
        return this._innerApiCalls.listDevices(request, options, callback);
    }
    /**
     * Equivalent to {@link listDevices}, but returns a NodeJS Stream object.
     *
     * This fetches the paged responses for {@link listDevices} continuously
     * and invokes the callback registered for 'data' event for each element in the
     * responses.
     *
     * The returned object has 'end' method when no more elements are required.
     *
     * autoPaginate option will be ignored.
     *
     * @see {@link https://nodejs.org/api/stream.html}
     *
     * @param {Object} request
     *   The request object that will be sent.
     * @param {string} request.parent
     *   Required. The device registry path. Required. For example,
     *   `projects/my-project/locations/us-central1/registries/my-registry`.
     * @param {number[]} request.deviceNumIds
     *   A list of device numeric IDs. If empty, this field is ignored. Maximum
     *   IDs: 10,000.
     * @param {string[]} request.deviceIds
     *   A list of device string IDs. For example, `['device0', 'device12']`.
     *   If empty, this field is ignored. Maximum IDs: 10,000
     * @param {google.protobuf.FieldMask} request.fieldMask
     *   The fields of the `Device` resource to be returned in the response. The
     *   fields `id` and `num_id` are always returned, along with any
     *   other fields specified.
     * @param {google.cloud.iot.v1.GatewayListOptions} request.gatewayListOptions
     *   Options related to gateways.
     * @param {number} request.pageSize
     *   The maximum number of devices to return in the response. If this value
     *   is zero, the service will select a default size. A call may return fewer
     *   objects than requested. A non-empty `next_page_token` in the response
     *   indicates that more data is available.
     * @param {string} request.pageToken
     *   The value returned by the last `ListDevicesResponse`; indicates
     *   that this is a continuation of a prior `ListDevices` call and
     *   the system should return the next page of data.
     * @param {object} [options]
     *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
     * @returns {Stream}
     *   An object stream which emits an object representing [Device]{@link google.cloud.iot.v1.Device} on 'data' event.
     */
    listDevicesStream(request, options) {
        request = request || {};
        const callSettings = new gax.CallSettings(options);
        return this._descriptors.page.listDevices.createStream(this._innerApiCalls.listDevices, request, callSettings);
    }
    // --------------------
    // -- Path templates --
    // --------------------
    /**
     * Return a fully-qualified location resource name string.
     *
     * @param {string} project
     * @param {string} location
     * @returns {string} Resource name string.
     */
    locationPath(project, location) {
        return this._pathTemplates.locationPathTemplate.render({
            project,
            location,
        });
    }
    /**
     * Parse the project from Location resource.
     *
     * @param {string} locationName
     *   A fully-qualified path representing Location resource.
     * @returns {string} A string representing the project.
     */
    matchProjectFromLocationName(locationName) {
        return this._pathTemplates.locationPathTemplate.match(locationName).project;
    }
    /**
     * Parse the location from Location resource.
     *
     * @param {string} locationName
     *   A fully-qualified path representing Location resource.
     * @returns {string} A string representing the location.
     */
    matchLocationFromLocationName(locationName) {
        return this._pathTemplates.locationPathTemplate.match(locationName)
            .location;
    }
    /**
     * Return a fully-qualified registry resource name string.
     *
     * @param {string} project
     * @param {string} location
     * @param {string} registry
     * @returns {string} Resource name string.
     */
    registryPath(project, location, registry) {
        return this._pathTemplates.registryPathTemplate.render({
            project,
            location,
            registry,
        });
    }
    /**
     * Parse the project from Registry resource.
     *
     * @param {string} registryName
     *   A fully-qualified path representing Registry resource.
     * @returns {string} A string representing the project.
     */
    matchProjectFromRegistryName(registryName) {
        return this._pathTemplates.registryPathTemplate.match(registryName).project;
    }
    /**
     * Parse the location from Registry resource.
     *
     * @param {string} registryName
     *   A fully-qualified path representing Registry resource.
     * @returns {string} A string representing the location.
     */
    matchLocationFromRegistryName(registryName) {
        return this._pathTemplates.registryPathTemplate.match(registryName)
            .location;
    }
    /**
     * Parse the registry from Registry resource.
     *
     * @param {string} registryName
     *   A fully-qualified path representing Registry resource.
     * @returns {string} A string representing the registry.
     */
    matchRegistryFromRegistryName(registryName) {
        return this._pathTemplates.registryPathTemplate.match(registryName)
            .registry;
    }
    /**
     * Return a fully-qualified device resource name string.
     *
     * @param {string} project
     * @param {string} location
     * @param {string} registry
     * @param {string} device
     * @returns {string} Resource name string.
     */
    devicePath(project, location, registry, device) {
        return this._pathTemplates.devicePathTemplate.render({
            project,
            location,
            registry,
            device,
        });
    }
    /**
     * Parse the project from Device resource.
     *
     * @param {string} deviceName
     *   A fully-qualified path representing Device resource.
     * @returns {string} A string representing the project.
     */
    matchProjectFromDeviceName(deviceName) {
        return this._pathTemplates.devicePathTemplate.match(deviceName).project;
    }
    /**
     * Parse the location from Device resource.
     *
     * @param {string} deviceName
     *   A fully-qualified path representing Device resource.
     * @returns {string} A string representing the location.
     */
    matchLocationFromDeviceName(deviceName) {
        return this._pathTemplates.devicePathTemplate.match(deviceName).location;
    }
    /**
     * Parse the registry from Device resource.
     *
     * @param {string} deviceName
     *   A fully-qualified path representing Device resource.
     * @returns {string} A string representing the registry.
     */
    matchRegistryFromDeviceName(deviceName) {
        return this._pathTemplates.devicePathTemplate.match(deviceName).registry;
    }
    /**
     * Parse the device from Device resource.
     *
     * @param {string} deviceName
     *   A fully-qualified path representing Device resource.
     * @returns {string} A string representing the device.
     */
    matchDeviceFromDeviceName(deviceName) {
        return this._pathTemplates.devicePathTemplate.match(deviceName).device;
    }
    /**
     * Terminate the GRPC channel and close the client.
     *
     * The client will no longer be usable and all future behavior is undefined.
     */
    close() {
        if (!this._terminated) {
            return this.deviceManagerStub.then(stub => {
                this._terminated = true;
                stub.close();
            });
        }
        return Promise.resolve();
    }
}
exports.DeviceManagerClient = DeviceManagerClient;
//# sourceMappingURL=device_manager_client.js.map